import asyncio
import os
from pathlib import Path

import aiohttp
from gwss.logger import logger

async def download_file(url, dest_file: os.PathLike):
    """

    :param url: computed unpkg url
    :param dest_file: computed destination of downloaded file
    :return:
    """


    logger.debug(f"Downloading file {url} to {dest_file}")
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            if "content-disposition" in response.headers:
                header = response.headers["content-disposition"]
                file = dest_file
                with open(file, mode="w") as file:
                    while True:
                        chunk = await response.content.read()
                        if not chunk:
                            break
                        file.write(chunk)
                print(f"Downloaded file {file}")


import logging
import sys

import click
import colorlog

sout = click.get_text_stream('stdout')

log = logging.getLogger('GWSS')
log.setLevel(logging.DEBUG)
sout_handler = logging.StreamHandler(sout)
color_sout_handler = colorlog.StreamHandler(stream=sout)
sout_handler.setLevel(logging.DEBUG)
sout_formatter = logging.Formatter("%(asctime)s:%(name)s:%(levelname)s:%(message)s")
sout_handler.setFormatter(sout_formatter)
color_sout_handler.setFormatter(colorlog.ColoredFormatter(
    '%(cyan)s%(asctime)s%(reset)s:%(log_color)s%(levelname)s%(reset)s%(green)s:%(name)s%(reset)s:%(message)s',
    datefmt="%Y-%m-%d %H:%M:%S",
    log_colors={
        'DEBUG': 'light_red',
        'INFO': 'green',
        'WARNING': 'yellow',
        'ERROR': 'red',
        'CRITICAL': 'red,bg_white',
    }
                                                          ))

log.addHandler(color_sout_handler)
logger = log



from importlib.metadata import version as i_version
import sys
import os
from pathlib import Path
from pprint import pprint

import click
from click import Context

from gwss import gwss, resolver_config
from gwss.config import config
from gwss.resolver import resolve_pkg
from gwss.unpkg import Unpkg
from gwss.utilities import prepare_config, path_validation, squish_info, path_create


class Site(object):
    last_result = None
    dest_dir = None
    def __init__(self, destination_directory, last_result=None):
        self.dest_dir = os.path.abspath(destination_directory or '.')
        self.last_result = None or last_result

    def __enter__(self):
        self.last_result = self.last_result
    def __exit__(self, exc_type, exc_value, tb):
        #self.last_result
        pass
def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo("v{}".format(i_version('gwss')))
    ctx.exit()

@click.group(invoke_without_command=True, chain=True)
@click.option('--version', '-v', is_flag=True, callback=print_version,
              expose_value=False, is_eager=False)
@click.option('--dest-dir', '-d', default='.site', type=Path)

@click.pass_context
def cli(ctx, dest_dir, last_result=None):
    ctx.obj = ctx.with_resource(Site(destination_directory=dest_dir, last_result=last_result))
    pass

@cli.command(name='list', help="list the script and style names from the '{}' file".format(Path.joinpath(Path.home(), '.gwss')))
def ls():
    cfg = prepare_config(config)
    print(cfg)

@cli.command(help="get all urls and destination directories for styles and scripts")
@click.option('--dest-dir', '-d', default='.site', type=Path)
@click.pass_context
def resolve(ctx, dest_dir):
    resolve_rendered = {}
    for root, projects in config.items():
        for v_ in projects:
            package_dict = resolve_pkg(v_)
            resolve_rendered[v_] = package_dict
    Site.last_result = resolve_rendered
    return resolve_rendered

@cli.command()
@click.option('--dest-dir', '-d', default=Path('./site_files'), type=Path)
@click.pass_context
def download(_ctx, dest_dir: Path | os.PathLike):
    if Site.last_result is not None:
        result = Site.last_result
        if path_validation(dest_dir):
            pass
        else:
            path_create(dest_dir)
        click.echo(f'Downloading all needed scripts and styles to {dest_dir}')
        for project, dict_ in result.items():
            version = dict_['version']
            urls = dict_['urls']
            dir_ = dict_['dir']
            pprint(urls)
            for name, url_ in urls.items():
                pprint(f"{name} -> {url_}")
                name_file = name.split('.')[0]
                name_ext = name.split('.')[1]
                dest_file = Path(os.getcwd()).joinpath(dest_dir, f"{name_file}-{version}.{name_ext}")
                pprint(dest_file)
                file_ = Unpkg(project, version, dir_, name, name, None)
                file_.unpkg_dl(url_, dest_file)
    else:
        click.secho(f"No previous results to use, use 'gwss resolve download'", err=True)






import asyncio
from os import PathLike
from typing import Any

import click

from gwss.download import download_file
from gwss import resolver
from furl import furl, Fragment

from gwss.logger import logger


class Unpkg:

    base_url = furl('https://unpkg.com')

    def __init__(self,
                 package: str,
                 version: str,
                 dir_: PathLike | None,
                 name: PathLike,
                 filename: PathLike,
                 s_or_s: str | None) -> object:
        """

        Args:
            package : str
                The package name
            version : str
                The version number of the package
            dir_ : str | PathLike | None
                root (dir)ectory that downloadable files are in (umd|dist|.)
                If isn't needed (if results are already pulled) should be left out
            name : str | PathLike
                The 'nickname' given to the file to save as
            filename : str | PathLike
                The first parent directory and filename of wanted file (without extension)
            s_or_s : str
                The string 'scripts' or 'styles'




        """
        self.package = package
        self.version = version
        self.dir_ = dir_
        self.name = name
        self.filename = filename
        self.s_or_s = s_or_s
        self.url = ''


    def unpkg_dl(self, url, dest_file: PathLike) -> bool | Any:
        """
        Download the package file from unpkg.com
        Parameters
        ----------
        url : furl | str | None
            url to download from
        dest_file : str | PathLike
            local file to download to

        Returns
        -------
        True if files are successfully downloaded
        False otherwise
        """
        logger.debug(f"Downloading files from Unpkg")
        logger.debug(f'Trying to download {self.unpkg_url()} to {dest_file}')
        asyncio.run(download_file(url=self.unpkg_url().url, dest_file=dest_file))


    def unpkg_url(self) -> furl:
        """
        Create a URL using the base URL, package, version, directory, file,
        and extension.

        :return:
        :param args: Positional arguments for directory and file.
        :param kwargs: Keyword arguments for directory and file.
        :return: The created URL as a string.
        """
        url = furl(self.base_url)
        # create url
        package = self.package
        url = url.add(path=f"{package}")
        version = self.version
        url = furl(f'{url.url}@{version}')
        _dir = self.dir_
        url = url.add(path=f"{_dir}")
        file = self.filename
        url = url.add(path=f"{file}")
        self.url = url.path.normalize()
        return url


from typing import Union
from lastversion import latest
def get_latest(package: str) -> str:
    version: str = latest(package, output_format='tag')
    if version[0] == 'v':
        version = version[1:]
    assert version is not None
    return version


#
# UNPKG Resolver
#
# return all data needed to download scripts and styles from UNPKG
import sys
from os import PathLike
from pprint import pprint
from typing import Tuple, Dict, Any

import click
import yaml
import pathlib
from pathlib import Path

from furl import furl

from gwss.resolver_config import projects
from gwss.unpkg import Unpkg
from gwss.versions import get_latest

def resolve_pkg(package: str) -> dict[Any, Any]:
    prelim_package_version = get_latest(package=package)
    assert projects[package] is not None
    package_dict = projects[package]
    script_urls, style_urls = dict(), dict()
    if projects[package].__contains__('scripts'):
        script_urls = resolve_links(package, prelim_package_version, 'script', package_dict['dir'], package_dict['scripts'])
    if projects[package].__contains__('styles'):
        style_urls = resolve_links(package, prelim_package_version, 'style', package_dict['dir'], package_dict['styles'])
    rendered_links = dict(script_urls, **style_urls)
    package_dict['version'] = prelim_package_version
    package_dict['urls'] = rendered_links
    if package_dict.__contains__('styles'):
        del package_dict['styles']
    if package_dict.__contains__('scripts'):
        del package_dict['scripts']
    return package_dict

def resolve_links(package, version: str, s_or_s: str, dir_: PathLike, links: dict):
    """
    Returns the full url for the unpkg files in 'links'
    Parameters
    ----------

    package : str
        name of package
    version : str
        the computed version of the package
    s_or_s : str
        literal text of 'script' or 'style'
    dir_ : PathLike
        first child directory the files is in, relative to root of package
    links : dict
        A list of files to resolve

    Returns
    -------
        urls for individual files
    """
    urls = dict()
    for name, filename in links.items():
        single_unpkg_file_url = Unpkg(package, version=version, dir_=dir_, filename=filename, name=name, s_or_s=s_or_s).unpkg_url()
        if s_or_s is not None:
            if s_or_s == 'script':
                urls[f"{name}.js"] = single_unpkg_file_url
            if s_or_s == 'style':
                urls[f"{name}.css"] = single_unpkg_file_url
    return urls